function [randfunc,randwts,basisf,nx,ny,xvals,optsused]=rbcmake(opts)
%
% [randfunc,randwts,basisf,nx,ny,xvals,optsused]=rbcmake(opts): makes and
% shows random band- and space-limited functions (cartesian separation)
%
%  opts.npts:  number of points on each side of array
%  opts.xvalmax: maximum x-value
%  opts.ranks: list of ranks (all basis functions of a rank are used;
%      rank n generates n+1 basis functions
%  opts.nexamples:  number of random examples to make
%  opts.colormaptype:  color map for plot
%  opts.plotbasisf:  nonzero to plot basis functions
%  opts.plotrandfunc:  nonzero to plot random functions
%  opts.nocross:  nonzero to suppress contributions of "mixed"
%      eigenfunctions with both nx and ny nonzero
%
%  randfunc[npts npts nexamples]: the random functions
%  randwts[length(nx) nexamples]: the random weights
%  basisf[npts npts length(nx)]: the basis functions
%  nx: the list of nx-indices for the cartesian decomposition
%  ny: the list of ny-indices for the cartesian decomposotion
%  xvals: the list of x-values
%  optsused: value of opts that are used (with defaults inserted)
%
%  uses monic Hermite polynomials, Hermite functions, orthonormalized
%     example:
%  opts=setfields([],{'ranks','npts','nexamples'},{[0 1 2],128,100});
%  [randfunc,randwts,basisf,nx,ny,xvals,optsused]=rbcmake(opts);
%
%   See also:  HERMCOEF, RBCFMAKE.
%
if (~isfield(opts,'npts')) opts.npts=256; end
if (~isfield(opts,'xvalmax')) opts.xvalmax=6; end
if (~isfield(opts,'ranks')) opts.ranks=[0 1 2];end;
if (~isfield(opts,'nexamples')) opts.nexamples=30;end;
if (~isfield(opts,'colormaptype')) opts.colormaptype='jet'; end
if (~isfield(opts,'plotbasisf')) opts.plotbasisf=1; end
if (~isfield(opts,'plotrandfunc')) opts.plotrandfunc=1; end
if (~isfield(opts,'nocross')) opts.nocross=0; end
optsused=opts;
%
npts=opts.npts;
xvalmax=opts.xvalmax;
nexamples=opts.nexamples;
ranks=unique(opts.ranks);
colormaptype=opts.colormaptype;
% calculate the Gaussian envelope
xvals=xvalmax*[(-npts+1)/2:(npts-1)/2]/(npts/2);
gxv=exp(-xvals.*xvals/4);
gaussian=gxv'*gxv;
%calculate the Hermite polynomials
ranks=unique(opts.ranks);
nx=[];ny=[];
for irank=1:length(ranks)
   if (opts.nocross==0)
      nx=[nx,[0:ranks(irank)]];
      ny=[ny,fliplr([0:ranks(irank)])];
   else %eliminate crossterms
      nx=[nx,unique([ranks(irank),0])];
      ny=[ny,fliplr(unique([ranks(irank),0]))];
   end
end
hvals=hermeval([0:max(ranks)],xvals);
%calculate the 2-d normalized basis functions
nbases=length(nx);
for ibase=1:nbases
   basisf(:,:,ibase)=(hvals(nx(ibase)+1,:)'*hvals(ny(ibase)+1,:)).*gaussian;
   basisf(:,:,ibase)=basisf(:,:,ibase)/sqrt(2.*pi*factorial(nx(ibase))*factorial(ny(ibase)));
end
%
%make random linear combinations
for iexamp=1:nexamples
   vrandwts=reshape(randn(nbases,1),[1 1 nbases]);
   vrandwts=vrandwts/sqrt(sum(vrandwts.^2)); %ensure unit length
   randfunc(:,:,iexamp)=sum(basisf.*repmat(vrandwts,[npts npts 1]),3);
   randwts(:,iexamp)=reshape(vrandwts,[nbases,1]);
end
%plot the basis functions
if (opts.plotbasisf>0)
   optsused.handle_basisf=figure;
	zm=max(max(max(abs(basisf))));
	[nr,nc]=nicesubp(nbases,[0.6 1.0]);
	for ibase=1:nbases
   	subplot(nr,nc,ibase);
   	colormap(colormaptype);
   	imagesc(basisf(:,:,ibase),[-zm zm]);
   	title(cat(2,'n_x n_y ',sprintf('%1.0f %1.0f',nx(ibase),ny(ibase))));
   	axis equal;axis tight;axis off;
   end
end
%plot random linear combinations
if (opts.plotrandfunc>0)
   optsused.handle_randfunc=figure;
	zm=max(max(max(abs(randfunc))));
	[nr,nc]=nicesubp(nexamples,[0.6 1.0]);
	for iexamp=1:nexamples
   	subplot(nr,nc,iexamp);
   	colormap(colormaptype);
   	imagesc(randfunc(:,:,iexamp),[-zm zm]);
   	axis equal;axis tight;axis off;
   end
end

