function [randfunc,randwts,basisf,mu,nu,desc,xvals,optsused]=rbpmake(opts)
%
% [randfunc,randwts,basisf,mu,nu,desc,xvals,optsused]=rbpmake(opts): makes and
% shows random band- and space-limited functions (polar separation)
%
%  opts.npts:  number of points on each side of array
%  opts.xvalmax: maximum x-value
%  opts.ranks: list of ranks (all basis functions of a rank are used;
%      rank n generates n+1 basis functions
%  opts.nexamples:  number of random examples to make
%  opts.colormaptype:  color map for plot
%  opts.plotbasisf:  nonzero to plot basis functions
%  opts.plotrandfunc:  nonzero to plot random functions
%  opts.muvals: if nonempty, then mu is restricted to this list
%  opts.nuvals: if nonempty, then nu is restricted to this list
%
%  randfunc[npts npts nexamples]: the random functions
%  randwts[length(nx) nexamples]: the random weights
%  basisf[npts npts length(nx)]: the basis functions
%  mu: the list of mu-indices for the cartesian decomposition
%  nu: the list of nu-indices for the cartesian decomposotion
%  desc: list of 'Z','R',or 'I' as in mixstim
%     'Z'->mu=0; 'R', 'I' for mu>0, cosine and sine parts, respectively
%  xvals: the list of x-values
%  optsused: value of opts that are used (with defaults inserted)
%
%  uses monic Hermite polynomials, Hermite functions, orthonormalized
%     example:
%  opts=setfields([],{'ranks','npts','nexamples'},{[0 1 2],128,100});
%  [randfunc,randwts,basisf,mu,nu,desc,xvals,optsused]=rbpmake(opts);
%
%   See also:  HERMCOEF, RBCMAKE.
%
if (~isfield(opts,'npts')) opts.npts=256; end
if (~isfield(opts,'xvalmax')) opts.xvalmax=6; end
if (~isfield(opts,'ranks')) opts.ranks=[0 1 2];end;
if (~isfield(opts,'nexamples')) opts.nexamples=30;end;
if (~isfield(opts,'colormaptype')) opts.colormaptype='jet'; end
if (~isfield(opts,'plotbasisf')) opts.plotbasisf=1; end
if (~isfield(opts,'plotrandfunc')) opts.plotrandfunc=1; end
if (~isfield(opts,'muvals')) opts.muvals=[]; end
if (~isfield(opts,'nuvals')) opts.nuvals=[]; end
optsused=opts;
%
npts=opts.npts;
xvalmax=opts.xvalmax;
nexamples=opts.nexamples;
ranks=unique(opts.ranks);
colormaptype=opts.colormaptype;
% calculate the Gaussian envelope
xvals=xvalmax*[(-npts+1)/2:(npts-1)/2]/(npts/2);
gxv=exp(-xvals.*xvals/4);
gaussian=gxv'*gxv;
%calculate the Hermite polynomials
ranks=unique(opts.ranks);
mu=[];nu=[];,desc=[];
for jrank=1:length(ranks)
   irank=ranks(jrank);
   muadd=sort(abs([-irank:2:irank]));
   muadd=sort(muadd);
   nuadd=(irank-muadd)/2;
   if (mod(irank,2)==0); descadd='Z'; else descadd=[]; end
   descadd=[descadd,repmat('RI',1,floor((irank+1)/2))];   
   include=ones(1,irank+1);
   if (~isempty(opts.muvals))
      for itest=1:(irank+1)
         if (~ismember(muadd(itest),opts.muvals)) include(itest)=0; end
      end
   end
   if (~isempty(opts.nuvals))
      for itest=1:(irank+1)
         if (~ismember(nuadd(itest),opts.nuvals)) include(itest)=0; end
   	end
   end
   mu=[mu,muadd(find(include==1))];
   nu=[nu,nuadd(find(include==1))];
   desc=[desc,descadd(find(include==1))];
end
%
theta=atan2(repmat(xvals',[1,npts]),repmat(xvals,[npts 1]));
radsq=repmat(xvals'.^2,[1,npts])+repmat(xvals.^2,[npts 1]);
rvals=reshape(radsq,[1 npts*npts]);
nbases=length(mu);
for ibase=1:nbases
   radvals=reshape(hgeneval(mu(ibase),nu(ibase),rvals),[npts npts]);
   basis=gaussian.*(radsq.^(mu(ibase)/2)).*radvals.*exp(i*mu(ibase)*theta);
   basis=basis/sqrt(2^(mu(ibase)+2*nu(ibase)).*pi*factorial(mu(ibase)+nu(ibase))*factorial(nu(ibase)));
   if (desc(ibase)=='Z') basisf(:,:,ibase)=basis/sqrt(2); end
   if (desc(ibase)=='R') basisf(:,:,ibase)=real(basis); end
   if (desc(ibase)=='I') basisf(:,:,ibase)=imag(basis); end
end
%
%make random linear combinations
for iexamp=1:nexamples
   vrandwts=reshape(randn(nbases,1),[1 1 nbases]);
   vrandwts=vrandwts/sqrt(sum(vrandwts.^2)); %ensure unit length
   randfunc(:,:,iexamp)=sum(basisf.*repmat(vrandwts,[npts npts 1]),3);
   randwts(:,iexamp)=reshape(vrandwts,[nbases,1]);
end
%plot the basis functions
if (opts.plotbasisf>0)
   optsused.handle_basisf=figure;
	zm=max(max(max(abs(basisf))));
	[nr,nc]=nicesubp(nbases,[0.6 1.0]);
	for ibase=1:nbases
   	subplot(nr,nc,ibase);
   	colormap(colormaptype);
   	imagesc(basisf(:,:,ibase),[-zm zm]);
      title(cat(2,'\mu \nu ',sprintf('%1.0f %1.0f',mu(ibase),nu(ibase))));
   	axis equal;axis tight;axis off;
   end
end
%plot random linear combinations
if (opts.plotrandfunc>0)
   optsused.handle_randfunc=figure;
	zm=max(max(max(abs(randfunc))));
	[nr,nc]=nicesubp(nexamples,[0.6 1.0]);
	for iexamp=1:nexamples
   	subplot(nr,nc,iexamp);
   	colormap(colormaptype);
   	imagesc(randfunc(:,:,iexamp),[-zm zm]);
   	axis equal;axis tight;axis off;
   end
end

